"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Error which should be thrown by a repository when it cannot find an aggregate root
 */
class AggregateNotFoundError extends Error {
    constructor(id) {
        super(`Aggregate ${id} not found or deleted`);
    }
}
exports.AggregateNotFoundError = AggregateNotFoundError;
/**
 * Error which should be thrown by a repository when an aggregate cannot be saved
 * because it had been saved already by another process
 */
class ConcurrencyError extends Error {
    constructor(id, currentVersion, desiredVersion) {
        super(`Cannot save aggregate ${id}. It has already been modified`);
        this.reason = `Trying to update version ${currentVersion} to ${desiredVersion}`;
    }
}
exports.ConcurrencyError = ConcurrencyError;
/**
 * A generic error which should be thrown by aggregate root when a command cannot be processed
 * either due to do it being incomplete or attempting to put the aggregate in an illegal state
 */
class DomainError extends Error {
    constructor(id, title, reason) {
        super(`Cannot modify aggregate ${id} into an invalid state`);
        this.title = title;
        this.reason = reason;
    }
}
exports.DomainError = DomainError;
