"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const logger = debug_1.default('fun-ddr:events');
const handlers = new Map();
async function runHandler(handler, ev) {
    const handlerLog = logger.extend(ev.name);
    handlerLog(`Calling handler ${handler.name || 'anonymous'}(${ev.id})`);
    try {
        return await handler(ev);
    }
    catch (e) {
        handlerLog.extend('error')('Error in handler %O', e);
    }
}
async function handlersAndChildren(name, promises) {
    const resolved = await Promise.all(promises);
    const childPromises = resolved.filter(value => typeof value === 'object' && value !== null && 'then' in value);
    if (childPromises.length > 0) {
        await handlersAndChildren(name, childPromises);
    }
}
function handle(name, handler) {
    logger(`Adding handler for event ${name}: ${handler.name}`);
    const handlersOfEvent = handlers.get(name) || [];
    handlersOfEvent.push(handler);
    handlers.set(name, handlersOfEvent);
    return (id, data) => handler({
        id,
        data,
        name,
    });
}
exports.handle = handle;
function emit(name, ev) {
    logger(`Emitting event ${ev.name} for resource ${ev.id}`);
    const handlersOfEvent = (handlers.get(name) || []);
    const promises = handlersOfEvent.map(handle => runHandler(handle, ev));
    return handlersAndChildren(name, promises);
}
exports.emit = emit;
function emitImmediate(id, name, data) {
    return emit(name, {
        id,
        name,
        data,
    });
}
exports.emitImmediate = emitImmediate;
